;
; boot.asm - IDEDOS booter
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;***** Header for the cartridge *****
;OP    Must be in open_cfg
;LW    $8000-$9fff
;NO    No open_cfg

		.word cart_reset,cart_restore
		.byte $c3,$c2,$cd,$38,$30
		.byte $20
;---------------------------
		.text "IDE64"
copyright	.text " COPYRIGHT 1998-2004",$0d
		.text "J. SOUCEK, "
		.text "T. PRIBYL, "
		.text "ZS. KAJTAR "
		.byte $00
chksum		.word 0
uuid		.include uuid
;---------------------------
;Boot
;
cart_reset	sei
		sta cfg_8
		ldx #0
		stx $dc03
		dex
		stx $dc02
                txs
		lda #%11111101
		sta $dc00
dye		dex
		beq th
		lda $dc01
		cmp $dc01
		bne dye
		eor #%01111111		;shift
		bne th
		sta $dc02
		sta $dc03
		sta $dc00
		ldx #5
nn		lda $d011		;Wait stable machine state
		bpl *-3
		lda $d011
		bmi *-3
		dex
		bne nn
                bit $d0bc		;Test for cpu type
		.if SUPERCPU=1
		bmi selftest
		.else
		bpl selftest
		.fi
		sta cfg_open		;to be sure, reset buttons are not ideal
		jsr OP_check_sign	;First boot/reset?
		bne selftest
		jsr OP_external_init
		lda $d030		;C128?
		lsr
		bcs sss
		lda $d011
		bmi *-3
		lda $d011
		bpl *-3
		lda #20
		cmp $d012		;PAL/NTSC?
		bcs *-3
		lda $d011
		and #128
		jsr crtc_init
sss		dec $318
                .if SUPERCPU=1
                ;stz $00
                lda $d246
                clc
                adc #5
                tcs
       		lda $d241	;PCH
                pha
                lda $d240	;PCL
                pha
                lda $d24e	;SR
                pha
                lda $d248	;A
                pha
                lda $d24a	;X
                pha
                lda $d24c	;Y
                pha
                .else
		#psr $fce3                
		lda #%00100100
		pha
                ldx #$f9
		txs
                .fi
		lda #BANK+128
		jmp monitor

th		sta cfg_8
		jsr LWNO_init_io	;$fda3

		lda #0
		tay
zacyk		sta $0002,y		;Clear memory
		sta $0200,y
		sta $0300,y
		iny
		bne zacyk

		ldy #<$033c
		sty $b2
		ldy #>$033c
		sty $b3
		ldy #$a0
		sty $c2
		tax
		jsr $fd8c		;Set memory pointers

		jsr LWNO_set_old314vect
		jsr kill_nmi		;No RUN/STOP+RESTORE until fully initialized

		jsr LWNO_init_vic	;$e518
                jsr exit_c128key
		jsr $e544		;clrscr

		lda $ff5e		;kernal rev 1?
                eor #$aa
                beq *+5
		jsr $ff5e		;Machine type, etc.
		lda #7
		sta $dd00

		lda $d030		;C128?
		lsr
		bcs n128
		lda $2a6
		jsr crtc_init
n128
		cli

		.if SUPERCPU=1
                lda #<$e502
                sta $b0
                lda #>$e502		;JiffyDOS F-keys patch
                sta $b1
                .else
		jsr LWNO_detect_jiffydos
                .fi

		jsr $e453		;Init $0300 vectors
		jsr $e3bf		;Init basic

		lda #$49		;eor #$30
		sta $84			;cmp #$10
		sta $88			;eor #$30
		lda #$30		;patch chrget/chrgot, to make it 2 cycles faster ;)
		sta $85
		sta $89
		lda #$c9
		sta $86
		lda #$10
		sta $87
                
		lda $2b
		ldy $2c
		jsr $a408		;check memory overlap
		jsr $a644		;basic NEW

		.if DE32_EXISTS=1
		lda ideconf_port
		cmp #$12		;wrong dos for cartridge without
		bne node32              ;cartconfig register
		.fi

		bit $d0bc		;Teszt for cpu type
		.if SUPERCPU=1
		bpl cpu_type_ok
		.else
		bmi cpu_type_ok
		.fi
;---------------------------
;Wrong CPU
;
node32		jsr LWNO_set_old314vect
		jsr LW_set_screen_cold	;not the right cpu for this dos

		jsr $e429
		jmp cart_reset_end	;sorry, wrong cpu
;---------------------------
;ROM fix
;
		.if SUPERCPU=1
memtrashpatch	lda ($c3),y
		bcc *+6
		lda $314,y

		.if EXPERIMENTAL=1
brkpatch     	.word r_brk,r_brk,$ff48;r_brk
		.else
brkpatch        .word r_brk,r_brk,r_brk
                .fi

jiffypatch	jsr $ffc3
		.byte $80

		.if EXPERIMENTAL=1
irqpatch	sta scpu_hwdisable	;3	;irqroute
		sta $00			;2      ;irqroute
                pea irq_return		;3 8    ;irqroute
                lda #%00100100 		;2      ;irqroute
                pha			;1      ;irqroute
                lda $00			;2      ;irqroute
                sta cfg_stnd		;3      ;irqroute
		bra irqpatch		;2      ;irqroute
                .fi
                
		.fi
;---------------------------
;Right CPU
;
cpu_type_ok	jsr LWNO_set_new314vect
		jsr kill_nmi
		jsr LW_basic_init
		sta cfg_open
		.if SUPERCPU=1
		jsr scpu_default_opt
                .fi

		jsr OP_external_init
		#lda0
		sta monitor_ripper_exp

		.if SUPERCPU=1		
		ldx #6
lu		lda memtrashpatch,x     ;patch kernal to not trash memory
		sta $1fd20,x
		dex
		bpl lu
                
                ldx #5
lu2             lda brkpatch,x		;brk with hwreg enabled?
                sta $17ffa,x
                dex
                bpl lu2
                
                ldx #3
lu3             lda jiffypatch,x	;JiffyDOS CONTROL+D hack
		sta $162a6,x
                dex
                bpl lu3

		.if EXPERIMENTAL=1
                ldx #17					;irqroute
lu4             lda irqpatch,x		;IRQ patch      ;irqroute
		sta $17f45,x                            ;irqroute
                dex                                     ;irqroute
                bpl lu4                                 ;irqroute
                .fi

		.fi

		jsr OP_check_sign	;First boot/reset?
		beq ini_allready
;---------------------------
;Poweron detected
;
ini_new_call	jsr kill_drives		;No IDE devices
		lda setup_boot_drive_exp
		sta monitor_dr_exp
		#sta_cfg_16

		sei
		lda #$80
		sta msgmode
		sta $291		;Shift+C= disabled
		.if SUPERCPU=0
		lda #<r_brk
		sta $fffa
		sta $fffe
		lda #>r_brk
		sta $fffb
		sta $ffff
		.fi
		lda #51
		sta $1
		ldx #0
hthee		lda $d000,x
		sta $2000,x
;                lsr
;                ora $d000,x
;                sta $2400,x
                lda $d100,x
		sta $2100,x
                lda $d800,x
                sta $2200,x
		lda logo,x
		sta $2300,x
		inx
		bne hthee
		lda #55
		sta $1
		stx $d020
		stx $d021		;Black screen
		lda #24
		sta $d018
                jsr LWNO_print_imediate ;cfg_16 after this
                .text 154,224,225,226,227,228,229,230,231,232,233
                .text 13,234,235,236,237,238,239,240,241,242,243,0

		lda #<copyright
		ldy #>copyright
		jsr LWNO_print_ay
		lda #27
		sta $d011

		sei
                jsr romtest
                beq vx

		jsr LWNO_print_imediate
		.text 13,13,158,"CORRUPT PEROM!",0
                jmp *

vx		#sta_cfg_open
		jsr OP_set_sign		;Now we fully initialized...
		jsr LWNO_print_imediate ;cfg_16 after this
		.text 13,13,"IDEDOS ","C"+128,"O"+128,"M"+128,"E"+128,"S"+128," WITH ABSOLUTELY NO WARRANTY"
		.text "T","H"+128,"I"+128,"S"+128,"  ","I"+128,"S"+128,"  ","F"+128,"R"+128,"E"+128,"E"+128,"  "
                .text "S"+128,"O"+128,"F"+128,"T"+128,"W"+128,"A"+128,"R"+128,"E"+128,",  ","A"+128,"N"+128,"D"+128,"  "
                .text "Y"+128,"O"+128,"U"+128,"  ","A"+128,"R"+128,"E"+128
		.text "W"+128,"E"+128,"L"+128,"L"+128,"C"+128,"O"+128,"M"+128,"E"+128,"   "
                .text "T"+128,"O"+128,"   ","R"+128,"E"+128,"D"+128,"I"+128,"S"+128,"T"+128,"R"+128,"I"+128,"B"+128,"U"+128,"T"+128,"E"+128,"  "
                .text "I"+128,"T"+128,"   ","U"+128,"N"+128,"D"+128,"E"+128,"R"+128
		.text "C"+128,"E"+128,"R"+128,"T"+128,"A"+128,"I"+128,"N"+128," "
                .text "C"+128,"O"+128,"N"+128,"D"+128,"I"+128,"T"+128,"I"+128,"O"+128,"N"+128,"S"+128,". S","E"+128,"E"+128," "
                .text "COPYING! (GPL-2)"
		.text 13,"B","U"+128,"I"+128,"L"+128,"T"+128,": "
		.include date
		.text 13,13,"P","L"+128,"E"+128,"A"+128,"S"+128,"E"+128," "
                .text "W"+128,"A"+128,"I"+128,"T"+128,", ","D"+128,"E"+128,"T"+128,"E"+128,"C"+128,"T"+128,"I"+128,"N"+128,"G"+128," "
                .text "D"+128,"E"+128,"V"+128,"I"+128,"C"+128,"E"+128,"S"+128,"...",13,13,0

		.if (SUPERCPU=1) & (EXPERIMENTAL=1)
                sta scpu_hwdisable	;irqroute
                .fi
		jsr LWNO_set_new314vect	;Enable RUN/STOP+RESTORE

		lda $d030		;C128?
		lsr
		bcs n128_		;no
		jsr crtc_logo
n128_
		jsr bashdinit
		#sta_cfg_open
                
		asl msgmode		;clear messages

cs		lda $dc01
		cmp #%11111011
                beq cs

		lda setup_atribut_exp	;Test BOOT Power up
		bpl no_load_boot

read_c_key	lda $dc01
		cmp #%01111111		;Test 'RUN/STOP'
		beq no_load_boot
		cmp #%11011111		;Test 'C='
		beq no_load_boot

		ldx #12
		stx ActualFNLength
ve		lda bf-1,x
		sta $1ff,x
		dex
		bne ve
		stx ActualFilename
		lda #2
		sta ActualSecAddr
		sta ActualFilename+1

		#psr loadback
		#psr switch_16k
		#lda0
		jmp newload

run		.text "R","U"+128,13
bf		.text "1//:BOOT,PRG"

loadback	#sta_cfg_open
		bcs no_load_boot
		lda Status
		cmp #$40
		bne no_load_boot
		stx 45
		sty 46			;set basic end pointer
		ldx #3
		stx 198
vt		lda run-1,x
		sta 630,x
		dex
		bne vt
		geq no_load_boot
;---------------------------
;Simple reset
;
ini_allready	lda #nop_d
		sta low_reg		;Interrupt current HDD command
		sta feat_g		;for atapi
		sta comm_g		;if there was any

		jsr OP_hdinit_drivenumbers

		ldx #0			;Needed, because powermanagement
vcaa1		lda status_g		;not works if there was an not
		lda low_reg		;aborted operation...
		bpl cmzz
		lda $d012
		cmp $d012		;We don't wait too long
		beq *-3			;Status takes for my hdd ca. 3 rasterlines
		dex
		bne vcaa1
cmzz

		lda setup_atribut_exp	;Test BOOT Always
		cmp #%11000000
		bcs read_c_key

no_load_boot    lda setup_atribut_exp	;Test SET BASIC CLOCK ENABLE/DISABLE
		and #CMOS_SET_TI
		beq skip_basic_clock
		jsr OP_set_basic_clock

skip_basic_clock
		.if LCD_DSPL=1
		jsr lcd_display_init
		.fi

		lda setup_atribut_exp	;Test keyboard repeat ENABLE/DISABLE
		and #CMOS_KEYREPEAT
		beq skip_key_rep
		lda #128
		sta 650			;Keyboard repeat enable
skip_key_rep
		jsr LWOP_init_c128key

                lda setup_chars_exp
                sta 646

		lda #147
		jsr newchrout_t
		lda $d011
		bpl *-3
		lda $d011
		bmi *-3
		.if (SUPERCPU=1) & (EXPERIMENTAL=1)
                sta scpu_hwdisable	;irqroute
                .fi
		jsr LWNO_set_new314vect	;Enable RUN/STOP+RESTORE
		#sta_cfg_open
		jsr LWOP_set_screen_col

		#sta_cfg_16
		lda #<$e474
		ldy #>$e474
		jsr LWNO_print_ay
		jsr LWNO_print_imediate

		.if SUPERCPU=1
		.text 157,"IDE-DOS "
		#version
		.text "S"
		.else
		.text "IDE-DOS "
		#version
		.fi
		.byte " EXPANDED",13,9,0

		lda status_g		;Last chance for status...

cart_reset_end	#psro $e39d
		jmp exit2basic		;Return to basic

;---------------------------
;IDE64 logo
;
logo		.byte $f9,$f9,$f9,$f9,$f9,$f9,$f9,$f9
		.byte $ff,$ff,$ff,$f0,$f0,$f0,$f0,$f0
		.byte $fc,$ff,$ff,$1f,$0f,$0f,$0f,$0f
		.byte $1f,$1f,$9f,$9f,$9f,$9f,$9f,$9f
		.byte $ff,$ff,$ff,$00,$00,$00,$ff,$ff
		.byte $c1,$c7,$cf,$0f,$0f,$0f,$cf,$cf
		.byte $ff,$ff,$ff,$80,$80,$80,$ff,$ff
		.byte $e0,$f8,$fc,$7c,$00,$00,$e0,$f8
		.byte $01,$03,$07,$0f,$1e,$3c,$78,$f0
		.byte $fc,$fc,$fc,$7c,$7c,$7c,$7c,$7c
		.byte $f9,$f9,$f9,$f9,$f9,$f9,$f9,$00
		.byte $f0,$f0,$f0,$f0,$ff,$ff,$ff,$00
		.byte $0f,$0f,$0f,$1f,$ff,$ff,$fc,$00
		.byte $9f,$9f,$9f,$9f,$9f,$1f,$1f,$00
		.byte $ff,$00,$00,$00,$ff,$ff,$ff,$00
		.byte $cf,$0f,$0f,$0f,$cf,$c7,$c1,$00
		.byte $ff,$80,$80,$80,$ff,$ff,$ff,$00
		.byte $fc,$7c,$7c,$7c,$fc,$f8,$e0,$00
		.byte $e0,$ff,$ff,$ff,$00,$00,$00,$00
		.byte $7c,$ff,$ff,$ff,$7c,$7c,$7c,$00
;---------------------------
;Old NMI routine and cfg16/cfg8 ???
;
cart_restore	cld
		.if (release=0) & (pprog=1);No perom programmer in released version
		jmp peromprg
		.fi
		sec
		php
		sta cfg_16
		#jmps restore_no_cia
;---------------------------
;Set screen colors and font
;
LWOP_set_screen_col .proc
		lda setup_paper_border_exp
                and #$0f
		cmp setup_chars_exp
		beq LW_set_screen_cold	;Same paper and char color is invalid...

		sta $d021
		lda setup_paper_border_exp
                lsr
                lsr
                lsr
                lsr
		sta $d020
		lda setup_atribut_exp	;Test lowercase ENABLE/DISABLE
		and #CMOS_LOWERCASE
		cmp #CMOS_LOWERCASE
		lda setup_chars_exp
		bcc skip_lower
		ldx #22
		gcs ifw

LW_set_screen_cold .proc
		lda #6			;Default colors
		sta $d021
		lda #14
		sta $d020
skip_lower	.global
		ldx #20
ifw		.global
		sta $0286
		stx $d018
		lda #27
		sta $d011
		rts
		.pend
;---------------------------
;Manage kernal vectors
;
LWNO_set_old314vect .proc
		sec
		.byte $a9		;NO SKIP1!
LWNO_set_new314vect .proc
		clc
		ldy #0
		ldx #15
loop		bcs old
		lda new314vect,x
		beq old
		sta $314,y
		lda #$de		;All between $de60-$deff...
in		sta $315,y
		iny
		iny
		dex
		bpl loop
		rts

old		lda $fd30,y
		sta $314,y
		lda $fd31,y
		gne in			;High address never on zp

new314vect	.byte <r_save,<r_load,0,<r_clall,<r_getin,0
		.byte <r_chrout,<r_chrin,<r_clrchn,<r_chkout
		.byte <r_chkin,<r_close,<r_open,<r_nmi,<r_brk,0
		.pend
;---------------------------
;Manage basic vectors
;
LW_basic_init	.proc
		sec
		.byte SKIP1
LWNO_basic_exit	.proc
		clc
		ldy #5			;Restore basic vectors
loop		lda $e44b,y
		bcc old
		lda newbasic,y
old		sta $0304,y
		dey
		bpl loop
		rts

newbasic	.word r_mytoken
		.word r_mytprt
		.word r_mydoit
		.pend
;---------------------------
;Set TI$ from RTC
;
OP_set_basic_clock .proc
		jsr LWOP_read_clock
                jsr OP_save_zeropage
		lda hour_exp
		jsr bcd2bin
		sta szam
		#stz_a szam+1
		#stz_aa szam+2
		jsr mul_60
		lda min_exp
		jsr bcd2bin
		jsr sum
		jsr mul_60
		lda sec_exp
		jsr bcd2bin
		jsr sum
		jsr mul_60
		lda szam
		sta $a2
		lda szam+1
		sta $a1
		lda szam+2
		sta $a0
		jmp OP_restore_zeropage
		.pend
;---------------------------
;Create timestamp to [pom1]
;
LWOP_timestamp	.proc
		#phy_a
		jsr LWOP_read_clock
                #ply_a
                lda sec_exp
		jsr bcd2bin
                #sta_zp_y pom1
                iny
                lda min_exp
		jsr bcd2bin
                #sta_zp_y pom1
                iny
                lda year_exp
		jsr bcd2bin
                clc
                adc #20
                cmp #100
                bcc *+4
                sbc #100
                and #%00111111
                #sta_zp_y pom1
                iny
                lda date_exp
		jsr bcd2bin
                #sta_zp_y pom1
		
                lda hour_exp
		jsr bcd2bin
                jsr cc
                dey

                lda month_exp
		jsr bcd2bin

		asl
cc              asl
                asl
                asl
                pha
                asl
                asl
                #ora_zp_y pom1
                #sta_zp_y pom1
                dey
                pla
                and #%11000000
                #ora_zp_y pom1
                #sta_zp_y pom1
                rts
                .pend
;---------------------------
;Prevent RUN/STOP+RESTORE
;
kill_nmi	.proc
		lda #<block_nmi
		sta 792
		lda #>block_nmi
		sta 793
		rts
		.pend
;---------------------------
;Init IO ($fda3)
;
LWNO_init_io	.proc
		jsr $fda3
		ldy #0
		.if (SECOND_SID >> 8)=$d6
		lda $d030		;C128?
		lsr
		bcc *+5
		.fi
		sty SECOND_SID+$18	;Second SID
		lda #7
		sta $dd00		;Release serial bus
		rts
		.pend
;---------------------------
;C128 keyboard init
;
LWOP_init_c128key .proc
		.if DE32_EXISTS=1
		lda $d030
		lsr
		bcs exit_c128key
		lda setup_atribut_exp
		and #CMOS_C128KEYS
		beq exit_c128key
		ldx #<r_keyb
		lda #>r_keyb
		ldy #%11111000
		gne exit_c128key.in
		.fi
;---------------------------
;C128 keyboard exit
;
exit_c128key	.proc
		ldx #<$eb48
		lda #>$eb48
		ldy #$ff
in		stx $028f
		sta $0290
                sty $d02f
                rts
                .pend
;---------------------------
;Init VIC ($e518)
;
;Used before cpu detection!
;
LWNO_init_vic	.proc
		ldx #3
		stx ActualOutput	;Standard output
		inx
		stx $028b		;keyboard repeat
                ldx #$2f
tntj		lda vicdefault-$11,x
		bmi *+5
		sta $d001,x
                dex
		cpx #$10
		bne tntj
		lda $d011
		ora #$80
		sta $d011
		lda #0
		sta ActualInput		;Standard input
		sta $0291
		sta $cf
qep		sta $d000,x		;Initialize VIC
		dex
		bpl qep

		lda #$0a
		sta $0289		;Keyboard buffer size
		sta $028c		;Keyboard first repeat

		lda #$0c
		sta $cd			;timer for cursor
		sta $cc			;no cursor blink
		rts

vicdefault	.byte         $37,$00,$00,$00,$08,$00;d010
		.byte $80,$0f,$00,$00,$00,$00,$00,$00;d018
		.byte $80,$80,$01,$02,$03,$04,$00,$01;d020
		.byte $02,$03,$04,$05,$06,$07,$4c,$ff;d028
                .byte $fc		;d030 C128
		.pend
;---------------------------
;Init external variables
;
OP_external_init .proc
		lda #INFO_KERNAL
		sta buffer_info_exp+STDIN
		sta buffer_info_exp+STDOUT
		lda #INFO_IGNORE
		sta buffer_info_exp+IGNOREINOUT
		lda #3
		sta buffer_drive_exp+STDOUT;stdout
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		ldx #92
vs		lda newchrin_ram_from,x
		sta newchrin_ram_exp,x
		dex
		bpl vs
		.fi
		lda #STDIN
		sta buffer_input_exp	;Set In/Output to standard
		lda #STDOUT
		sta buffer_output_exp
		#lda0
		sta zp_valid_exp
		sta direct_write_exp	;no direct write
		sta kill_port		;Just to be sure...
		sta load_autostart_exp	;no autostart
		sta buffer_drive_exp+STDIN;stdin
		sta buffer_logical_exp+STDIN
		sta buffer_logical_exp+STDOUT
                .if (JIFFYDOS_ACCEL=1) & (SUPERCPU=0) & (JIFFYDOS_ACCEL_CHRINOUT=0)
                sta JiffyDos_internal_exp
                .fi
		ldx #MAX_OPEN_FILES
httnh		sta open_files_logical_exp-1,x;All files closed
		dex
		bne httnh
		ldx #MAX_BUFFERS
httnh2		sta buffer_logical_exp-1,x;All buffers free
		dex
		bne httnh2
		lda $02A6		;PAL(1)/NTSC(0)
		sta TV_machine_type_exp
                .if SUPERCPU=0
		lda $b0			;jiffydos
		sta JiffyDos_present_exp
                .fi
		jmp LWOP_get_cmos_setup
		.pend
;---------------------------
;Check JiffyDos (Z=1)
;
		.if SUPERCPU=0
LWNO_detect_jiffydos .proc
		ldy #250
		ldx #184
j11x		iny
		beq yes_jiffy		;JiffyDos found
j1x		inx
		beq no_jiffy		;No Jiffydos
		lda $e45f-185,x
		cmp jiffytext-251,y
		beq j11x
cce		cpy #251
		beq j1x
		dex
		dey
		gne cce
yes_jiffy	lda #<$f672		;Fix jiffydos F-keys
		sta $b0
		lda #>$f672
		sta $b1
no_jiffy	rts

jiffytext	.byte $4a,$49,$46,$46,$59
		.pend
                .fi
;---------------------------
;Check/set ram sign
;
OP_set_sign	.proc
		clc
		.byte SKIP1
OP_check_sign	.proc
		sec
		ldx #38
loop		lda uuid-3,x		;chksum!
		bcc set
		cmp ident_label_exp-1,x
		bne sorry
set		sta ident_label_exp-1,x
nx		dex
		bne loop                ;Z=1
sorry		rts
		.pend
